#include <fcntl.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define DEFAULT_SLEEP 600
#define CONFIG_FILE "/etc/tush.conf"
#define LINESIZE 80

/*
	Tunnel Shell
	Author: Brian Epstein <bepstein@ias.edu>
	Tue Feb 24 10:14:31 EST 2009
	$Id: tush.c 311 2009-02-24 15:18:55Z ep $
*/

int main (argc,argv)
int argc;
char *argv[];
{
	  char buf[LINESIZE];
	  FILE *f_config;
	   int i = 0;
	struct passwd *pwd;
	  char *ptr;
	   int sleep_time = DEFAULT_SLEEP;

	/* Let's get our config file sleep time */
	if (f_config = fopen(CONFIG_FILE,"r")) {
		/* we have opened the config file */
		while (!feof(f_config) && i < 2) {
			if (fgets(buf, LINESIZE - 1, f_config) == 0) break;
			buf[LINESIZE-1] = 0;
			if (buf[0] == 0 || buf[0] == '\n') continue;
			ptr = strchr (buf, '=');
			if (ptr) {
				ptr++;
				if ( !strncmp(buf, "SLEEPTIME", 9)) {
					while ( isspace(*ptr) ) ++ptr;
					sleep_time = atoi(ptr);
					printf("Sleeptime = %d from /etc/tush.conf\n",sleep_time);
					break;
				}
			}
		}
	}

	/* Let's see if our CLI sleeptime is valid */
	if (argc == 2) {
		sleep_time = atoi(argv[1]);
	}

	if (sleep_time < 60) {
		printf("Sleep time under 60 seconds or unknown\n");
		printf("Setting sleep time to %d\n",DEFAULT_SLEEP);
		sleep_time = DEFAULT_SLEEP;
	} else {
		printf("Setting sleep time to %d\n",sleep_time);
	}

	/* Let's get the /etc/passwd entry to determine homedir */
	pwd = (struct passwd *)malloc(sizeof(struct passwd *));
	pwd = getpwuid(getuid());
	if (pwd == NULL) {
		printf("FATAL: Cannot find /etc/passwd entry for uid = %d\n",getuid());
		exit(1);
	}
	printf("Pre-chroot euid=%d uid=%d\n",geteuid(),getuid());
	if (geteuid() == 0) {
		/* Let's jail ourselves to our home directory */
		printf("chrooting to %s\n",pwd->pw_dir);
		if (chroot(pwd->pw_dir)) {
			printf("FATAL: Cannot chroot to %s\n",pwd->pw_dir);
			exit(1);
		}
		/* Chroot is successful, so drop root privs */
		if (setuid(getuid()) == -1) {
			printf("FATAL: Cannot drop root privileges\n");
			exit(1);
		}
		/* Safety step, ensure we are inside our jail */
		if (chdir("/") == -1) {
			printf("FATAL: Cannot chdir to /\n");
			exit(1);
		}
	} else {
		printf("Must run setuid root\n");
		exit(1);
	}
	printf("Post-chroot euid=%d uid=%d\n",geteuid(),getuid());
	/* Let's just sleep a long time, and remind people who we are */
	while(1) {
		/*
		   sizeof(int) = 4 could be 2147483647 (not really unsigned)
		   but just incase sizeof(int) = 2 on some systems.  It doesn't
		   really matter.
		*/
		printf("sleeping for %d seconds as euid=%d uid=%d\n",sleep_time,geteuid(),getuid());
		sleep(sleep_time);
	}
	/* should never get here, but just incase */
	free(pwd);
	exit(0);
} /* end main */
